slopectmax <- read.csv("Kovacevic_slopectmax_tree.csv")

library(rotl)
library(caper)

taxa<- c("Acanthemblemaria hancocki", "Atta sexdens rubropulosa", "Bathygobius soporator", "Ceratitis capitata", "Ceratitis rosa", 
         "Clupea harengus", "Cydia pomonella", "Cyrtobagous salviniae", "Deuteraphorura acicindelis", "Deuteraphorura pseudobosnaria", "Dicentrarchus labrax", 
         "Drosophila melanogaster", "Drosophila subobscura", "Eurypanopeus abbreviatus", "Exosphaeroma antikraussi", "Exosphaeroma gigas", 
         "Exosphaeroma laeviusculum", "Folsomia candida", "Glossina pallidipes", "Hippolyte obliquimanus", "Hyale grandicornis", 
         "Hyale hirtipalma", "Hypogastrura assimilis", "Hypogastrura viatica", "Lepomis gibbosus", "Linepithema humile",
         "Menippe nodifrons", "Nylanderia fulva", "Oncorhynchus kitsutch", "Oncorhynchus mykiss", "Palaemon northropi",
         "Parablennius marmoreus", "Pycnopsyche guttifer", "Rutilus rutilus", "Salmo salar", "Salmo trutta", "Salvelinus fontinalis", "Solenopsis invicta",
         "Tenebrio molitor", "Xenylla humicola")
resolved_names <- tnrs_match_names(taxa, context_name="Animals")
my_tree<- tol_induced_subtree(ott_ids = resolved_names$ott_id)
plot(my_tree, no.margin=TRUE)

slopectmax$Species <- as.character(slopectmax$Species)
resolved_names <- tnrs_match_names(slopectmax$Species, context_name="Animals")

#Create a column in ctmaxdatabase that matches tree tip labels
slopectmax$tree.tip.label <- paste0(resolved_names$unique_name, "_ott", resolved_names$ott_id) 
slopectmax$tree.tip.label <- gsub(" ", "_", slopectmax$tree.tip.label)

my_tree<- tol_induced_subtree(ott_ids = subset(resolved_names, !is.na(ott_id))$ott_id)
plot(my_tree, no.margin=TRUE)

#Make unique nodes
my_tree$node.label <- c(1:length(my_tree$node.label))

#Create arbitrary branch lengths, using the arbitrary method of Grafen
my_tree <- compute.brlen(my_tree)

cdat <- comparative.data(data = slopectmax, phy = my_tree, names = "tree.tip.label",
                         na.omit = FALSE)

################################## z ####################################
################## Exclude T.leucotrata from the analysis, 40 species

m1 <- pgls(z ~ 1, lambda = "ML", data = cdat)
summary(m1)

##Table 2
m2 <- pgls(z ~ ResponseToTime, lambda = "ML", data = cdat)
summary(m2)

m3 <- pgls(z ~ Habitat, lambda = "ML", data = cdat)
summary(m3)

m4 <- pgls(z ~ Climate, lambda = "ML", data = cdat)
summary(m4)


#######################CTmin analyses #####################
slopectmin <- read.csv("Kovacevic_slopectmin_tree.csv")

taxa<- c("Ceratitis capitata", "Ceratitis rosa", "Cydia pomonella", "Cyrtobagous salviniae", "Deuteraphorura acicindelis", "Deuteraphorura pseudobosnaria", 
         "Drosophila melanogaster", "Folsomia candida", "Hypogastrura assimilis", "Hypogastrura viatica", "Linepithema humile",
         "Litopenaeus vannamei", "Locusta migratoria", "Nylanderia fulva", "Penaeus semisulcatus", "Sitobion avenae", "Solenopsis invicta",
         "Tenebrio molitor", "Tribolium castaneum", "Xenylla humicola")

resolved_names <- tnrs_match_names(taxa, context_name="Animals")
my_tree<- tol_induced_subtree(ott_ids = resolved_names$ott_id)
plot(my_tree, no.margin=TRUE)

slopectmin$Species <- as.character(slopectmin$Species)
resolved_names <- tnrs_match_names(slopectmin$Species, context_name="Animals")

#Create a column in ctmindatabase that matches tree tip labels
slopectmin$tree.tip.label <- paste0(resolved_names$unique_name, "_ott", resolved_names$ott_id) 
slopectmin$tree.tip.label <- gsub(" ", "_", slopectmin$tree.tip.label)

my_tree<- tol_induced_subtree(ott_ids = subset(resolved_names, !is.na(ott_id))$ott_id)
plot(my_tree, no.margin=TRUE)

#Make unique nodes
my_tree$node.label <- c(1:length(my_tree$node.label))

#Create arbitrary branch lengths, using the arbitrary method of Grafen
my_tree <- compute.brlen(my_tree)

cdat <- comparative.data(data = slopectmin, phy = my_tree, names = "tree.tip.label",
                         na.omit = FALSE)

################################## z' #######################################
################## Exclude T.leucotrata from the analysis + 2 outliers, 20 species

m5 <- pgls(z ~ 1, lambda = "ML", data = cdat)
summary(m5)

##Table 2
m6 <- pgls(z ~ ResponseToTime, lambda = "ML", data = cdat)
summary(m6)

m7 <- pgls(z ~ Habitat, lambda = "ML", data = cdat)
summary(m7)

m8 <- pgls(z ~ Climate, lambda = "ML", data = cdat)
summary(m8)